<dl class="form_buttons">
	<dt><label for="{name}">{label}</label></dt>
	<dd class="info">
		{info}
	</dd>
	<dd>
		<input type="reset" value="{reset_text:Reset}"/>
		<input type="submit" value="{submit_text:Submit}"/>
	</dd>
	{clear}
</dl>
