<dl>
	<dt><label for="{name}">{label}</label></dt>
	<dd class="info">
		{info}
	</dd>
	<dd>
		<input class="form_money_value" type="text" name="{name}" value="{content}"/>
		<select class="form_money_currency" name="{name}_currency">
			<option value="EUR">&euro; (EURO)</option>
		</select>
	</dd>
	{clear}
</dl>
