<html>
	<head>
		<title>Website</title>
		<link rel="stylesheet" type="text/css" href="{css_url}"/>
		<script type="text/javascript" src="{javascript_url}"></script>
	</head>
	<body>
		
		{content}
	
	</body>
</html>
