<dl>
	<dt><label for="{name}">{label}</label></dt>
	<dd class="info">
		{info}
	</dd>
	<dd>
		<input type="password" name="{name}" value="{content}"/>
		<input type="password" name="{name}_check" value="{content}"/>
	</dd>
	{clear}
</dl>

