<div class="box box_{class}">
	<div class="box_top_wrap">
		<div class="box_top">
			{top}
		</div>
		{clear}
	</div>
	<div class="box_middle_wrap">
		<div class="box_middle">
			{content}
		</div>
		{clear}
	</div>
	<div class="box_bottom_wrap">
		<div class="box_bottom">
			{bottom}
		</div>
		{clear}
	</div>
	{clear}
</div>
