/**
 * Styles for DatePicker
 */

.datepickerControl {
	border:				1px solid #999999;
	background-color:	#EEEEEE;
	padding:			5px;
}

.datepickerControl table {
	border: 			none;
	margin: 			0;
}

.datepickerControl td {
	text-align:			center;
	font-family:		sans-serif;
	font-size:			11px;
	padding:			1px;
	border: 			none;
}

.datepickerControl tr.monthLabel td {
	background-color:	#FFFFFF;
	border:				1px solid #999999;
	font-weight:		bold;
}

.datepickerControl tr.navigation td {
	cursor:				pointer;
}

.datepickerControl tr.navigation td:hover {
	text-decoration:	underline;
}

.datepickerControl tr.dayLabel td {
	/*
	border:				1px solid #C0D8EA;
	background-color:	#C0D8EA;
	*/
	border:				1px solid #003366;
	background-color:	#003366;
	color:				white;
	font-weight:		bold;
}

.datepickerControl td.day, .datepickerControl td.dayothermonth {
	cursor:				pointer;
	background-color:	#FFFFFF;
	border:				1px solid #EEEEEE;
	width:				2em;
}

.datepickerControl td.dayothermonth {
	color:				#999999;
	font-style:			italic;
}

.datepickerControl td.day:hover {
	background-color:	#EBE4C0;
} 

.datepickerControl td.weekend {
	background-color:	#CCCCCC;
	font-style:			italic;
}

.datepickerControl td.today {
	font-weight:		bold;
}

.datepickerControl td.hour {
	cursor:				pointer;
	background-color:	#FFFFFF;
	border:				1px solid #EEEEEE;
	width:				2em;
}

.datepickerControl td.minute {
	cursor:				pointer;
	background-color:	#FFFFFF;
	border:				1px solid #EEEEEE;
	width:				2em;
}

.datepickerControl td input,
.datepickerControl td button {
	font-size:			11px;
	padding:			0;
	border:				1px solid #999999;
	text-align:			center;
}

.datepickerControl td.ampm {
	cursor:				pointer;
	background-color:	#CCCCCC;
	border:				1px solid #EEEEEE;
	width:				2em;
}

.datepickerControl td.current {
	font-weight:		bold;
	background-color:	#EBC2C0;
}

.datepickerControl input.current {
	background-color:	#EBC2C0;
}

.datepickerControl td.current:hover {
	background-color:	#EBC2C0;
}
